'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactPlotly = require('react-plotly.js');

var _reactPlotly2 = _interopRequireDefault(_reactPlotly);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var OncoPrint = function (_React$PureComponent) {
  _inherits(OncoPrint, _React$PureComponent);

  function OncoPrint() {
    _classCallCheck(this, OncoPrint);

    return _possibleConstructorReturn(this, (OncoPrint.__proto__ || Object.getPrototypeOf(OncoPrint)).apply(this, arguments));
  }

  _createClass(OncoPrint, [{
    key: 'getData',
    value: function getData() {
      var _props = this.props,
          inputData = _props.data,
          padding = _props.padding,
          sampleColor = _props.sampleColor;


      var events = (0, _utils.aggregate)(inputData);
      var genes = (0, _utils.getSortedGenes)(inputData);
      var samples = (0, _utils.getSortedSamples)(inputData);

      var ratios = (0, _utils.getEventRatiosPerGene)(inputData, samples.length);
      var formatGenes = function formatGenes(list) {
        return list.map(function (gene) {
          return gene + ' (' + ratios[gene] + '%)';
        });
      };

      var base = 0;
      var bBackground = [];
      var tBackground = [];
      var xBackground = [];
      var yBackground = [];

      // Background is used to draw the matrix (genes * samples)
      samples.forEach(function (s) {
        bBackground.push.apply(bBackground, _toConsumableArray(Array(genes.length).fill(base)));
        tBackground.push.apply(tBackground, _toConsumableArray(Array(genes.length).fill(s)));
        xBackground.push.apply(xBackground, _toConsumableArray(Array(genes.length).fill(1)));
        yBackground.push.apply(yBackground, _toConsumableArray(formatGenes(genes)));
        base += 1;
      });

      var background = {
        base: bBackground.map(function (i) {
          return i + padding;
        }),
        hoverinfo: 'text',
        marker: {
          color: sampleColor
        },
        name: 'No alteration',
        text: tBackground,
        orientation: 'h',
        type: 'bar',
        x: xBackground.map(function (i) {
          return i - padding * 2;
        }),
        y: yBackground
      };

      var data = [background];
      Object.keys(events).forEach(function (key) {
        var aggr = events[key];

        var width = 0.4;
        if (aggr.type === 'CNA') {
          width = 0.8;
        } else if (aggr.type === 'EXP') {
          width = 0.6;
        }

        // where to draw a bar for this entry
        var indexes = aggr.events.map(function (e) {
          return e.sample;
        }).map(function (s) {
          return samples.findIndex(function (sample) {
            return sample === s;
          });
        });

        data.push({
          base: indexes.map(function (i) {
            return i + padding;
          }),
          hoverinfo: 'name',
          marker: {
            color: (0, _utils.getColor)(aggr.events[0])
          },
          name: (0, _utils.getDisplayName)(aggr.events[0]),
          orientation: 'h',
          type: 'bar',
          width: width,
          x: Array(aggr.events.length).fill(1).map(function (i) {
            return i - padding * 2;
          }),
          y: formatGenes((0, _utils.getGeneNames)(aggr.events))
        });
      });

      return data;
    }

    // eslint-disable-next-line class-methods-use-this

  }, {
    key: 'getLayout',
    value: function getLayout() {
      return {
        barmode: 'stack',
        hovermode: 'closest',
        xaxis: {
          showgrid: false,
          showticklabels: false,
          zeroline: false
        },
        yaxis: {
          fixedrange: true,
          showgrid: false,
          zeroline: false
        }
      };
    }
  }, {
    key: 'render',
    value: function render() {
      var otherProps = {};
      if (this.props.fullWidth) {
        otherProps.style = {
          width: '100%',
          height: '100%'
        };
        otherProps.useResizeHandler = true;
      }

      return _react2.default.createElement(_reactPlotly2.default, _extends({ data: this.getData(), layout: this.getLayout() }, otherProps));
    }
  }]);

  return OncoPrint;
}(_react2.default.PureComponent);

OncoPrint.defaultProps = {
  fullWidth: false,
  padding: 0.05,
  sampleColor: 'rgb(190, 190, 190)'
};
exports.default = OncoPrint;