'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _reactPlotly = require('react-plotly.js');

var _reactPlotly2 = _interopRequireDefault(_reactPlotly);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * The OncoPrint component is used to view multile genetic alteration events
 * through an interactive and zoomable heatmap. It is a React/Dash port of the
 * popular oncoPrint() function from the BioConductor R package.
 * Under the hood, the rending is done using Plotly.js built upon D3.
 * Plotly's interactivity allows the user to bind clicks and hovers to genetic
 * events, allowing the user to create complex bioinformatic apps or workflows
 * that rely on crossfiltering.
 * Read more about the component here:
 * https://github.com/plotly/react-oncoprint
 */
var OncoPrint = function (_PureComponent) {
    _inherits(OncoPrint, _PureComponent);

    // Constructor
    function OncoPrint(props) {
        _classCallCheck(this, OncoPrint);

        var _this = _possibleConstructorReturn(this, (OncoPrint.__proto__ || Object.getPrototypeOf(OncoPrint)).call(this, props));

        _this.state = {
            xStart: null,
            xEnd: null
        };

        _this.resetWindowing = _this.resetWindowing.bind(_this);
        _this.handleChange = _lodash2.default.debounce(_this.handleChange.bind(_this), 250);
        return _this;
    }

    // Reset windowing to user preset on init or data change


    _createClass(OncoPrint, [{
        key: 'resetWindowing',
        value: function resetWindowing(props) {
            var range = props.range;


            var xStart = void 0,
                xEnd = void 0;
            if (range.length === 2) {
                xStart = range[0];
                xEnd = range[1];
            } else {
                xStart = null;
                xEnd = null;
            }

            return { xStart: xStart, xEnd: xEnd };
        }

        // Handle plot events

    }, {
        key: 'handleChange',
        value: function handleChange(event) {
            if (!this.props.onChange) {
                return;
            }

            // CLick (mousedown) or hover (mousemove)
            if (event.points) {
                var eventType = void 0;
                if (event.event.type === "mousedown") {
                    eventType = 'Click';
                } else if (event.event.type === "mousemove") {
                    eventType = 'Hover';
                } else {
                    eventType = 'Other';
                }

                this.props.onChange({
                    eventType: eventType,
                    name: event.points[0].data.name,
                    text: event.points[0].text,
                    curveNumber: event.points[0].curveNumber,
                    x: event.points[0].x,
                    y: event.points[0].y
                });
            }
            // Zoom
            else if (event['xaxis.range[0]'] || event['xaxis.range']) {
                    this.setState({
                        xStart: event['xaxis.range[0]'] || event['xaxis.range'][0],
                        xEnd: event['xaxis.range[1]'] || event['xaxis.range'][1]
                    });
                    this.props.onChange({
                        eventType: 'Zoom',
                        xStart: event['xaxis.range[0]'] || event['xaxis.range'][0],
                        xEnd: event['xaxis.range[1]'] || event['xaxis.range'][1]
                    });
                }
                // Autozoom
                else if (event['xaxis.autorange'] === true) {
                        this.setState({
                            xStart: null,
                            xEnd: null
                        });
                        this.props.onChange({
                            eventType: 'Autoscale'
                        });
                    }
                    // Guard
                    else {
                            this.props.onChange(event);
                        }
        }
    }, {
        key: 'getData',


        // Fetch data
        value: function getData() {
            var _props = this.props,
                inputData = _props.data,
                padding = _props.padding,
                colorscale = _props.colorscale,
                backgroundcolor = _props.backgroundcolor;

            // OncoPrint equivalent of x, y

            var events = (0, _utils.aggregate)(inputData);
            var genes = (0, _utils.getSortedGenes)(inputData);
            var samples = (0, _utils.getSortedSamples)(inputData);
            var ratios = (0, _utils.getEventRatiosPerGene)(inputData, samples.length);

            var formatGenes = function formatGenes(list) {
                return list.map(function (gene) {
                    return gene + ' (' + ratios[gene] + '%)';
                });
            };

            var base = 0;
            var bBackground = [];
            var tBackground = [];
            var xBackground = [];
            var yBackground = [];

            // Background is used to draw the matrix (genes * samples)
            samples.forEach(function (s) {
                bBackground.push.apply(bBackground, _toConsumableArray(Array(genes.length).fill(base)));
                tBackground.push.apply(tBackground, _toConsumableArray(Array(genes.length).fill(s)));
                xBackground.push.apply(xBackground, _toConsumableArray(Array(genes.length).fill(1)));
                yBackground.push.apply(yBackground, _toConsumableArray(formatGenes(genes)));
                base += 1;
            });

            var background = {
                base: bBackground.map(function (i) {
                    return i + padding;
                }),
                hoverinfo: 'text',
                marker: {
                    color: backgroundcolor
                },
                name: 'No alteration',
                text: tBackground,
                orientation: 'h',
                type: 'bar',
                x: xBackground.map(function (i) {
                    return i - padding * 2;
                }),
                y: yBackground
            };

            var data = [background];
            Object.keys(events).forEach(function (key) {
                var aggr = events[key];

                // Resize width depending on the mutation type
                var width = 0.4;
                if (aggr.type === 'CNA') {
                    width = 0.8;
                } else if (aggr.type === 'EXP') {
                    width = 0.6;
                }

                // Mutations should have the original text on it, not the type of mutation
                var text_arr = aggr.events.map(function (event) {
                    return event.sample + '<br>' + (0, _utils.getDisplayName)(event);
                });

                // where to draw a bar for this entry
                var indexes = aggr.events.map(function (e) {
                    return e.sample;
                }).map(function (s) {
                    return samples.findIndex(function (sample) {
                        return sample === s;
                    });
                });

                data.push({
                    base: indexes.map(function (i) {
                        return i + padding;
                    }),
                    hoverinfo: 'text',
                    marker: {
                        color: (0, _utils.getColor)(aggr.events[0], colorscale)
                    },
                    name: (0, _utils.getDisplayName)(aggr.events[0]),
                    text: text_arr,
                    orientation: 'h',
                    type: 'bar',
                    width: width,
                    x: Array(aggr.events.length).fill(1).map(function (i) {
                        return i - padding * 2;
                    }),
                    y: formatGenes((0, _utils.getGeneNames)(aggr.events))
                });
            });

            return data;
        }

        // Fetch layout

    }, {
        key: 'getLayout',
        value: function getLayout() {
            var _props2 = this.props,
                showlegend = _props2.showlegend,
                showoverview = _props2.showoverview,
                width = _props2.width,
                height = _props2.height;
            var _state = this.state,
                xStart = _state.xStart,
                xEnd = _state.xEnd;

            // Get initial range

            var initialRange = [xStart, xEnd];

            var layout = {
                barmode: 'stack',
                hovermode: 'closest',
                showlegend: showlegend,
                xaxis: {
                    showgrid: false,
                    showticklabels: false,
                    zeroline: false,
                    range: initialRange,
                    automargin: true
                },
                yaxis: {
                    showgrid: false,
                    zeroline: false,
                    fixedrange: true,
                    automargin: true
                },
                margin: { t: 20, r: 20, b: 20 }
            };

            if (showoverview) {
                layout.xaxis.rangeslider = { autorange: true };
            }

            return { layout: layout, width: width, height: height };
        }

        // Set xStart and xEnd on load

    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
            var _resetWindowing = this.resetWindowing(this.props),
                xStart = _resetWindowing.xStart,
                xEnd = _resetWindowing.xEnd;

            this.setState({ xStart: xStart, xEnd: xEnd });
        }

        // Reset xStart and xEnd on data change

    }, {
        key: 'componentDidUpdate',
        value: function componentDidUpdate(prevProps, prevState) {
            if (this.props.data !== prevProps.data) {
                var _resetWindowing2 = this.resetWindowing(this.props),
                    xStart = _resetWindowing2.xStart,
                    xEnd = _resetWindowing2.xEnd;

                this.setState({ xStart: xStart, xEnd: xEnd });
            }
        }

        // Main

    }, {
        key: 'render',
        value: function render() {
            var id = this.props.id;


            var data = this.getData();

            var _getLayout = this.getLayout(),
                layout = _getLayout.layout,
                width = _getLayout.width,
                height = _getLayout.height;

            var other = {
                style: {
                    width: width,
                    height: height
                },
                useResizeHandler: true
            };

            return _react2.default.createElement(
                'div',
                { id: id },
                _react2.default.createElement(_reactPlotly2.default, _extends({
                    data: data,
                    layout: layout,
                    onClick: this.handleChange,
                    onHover: this.handleChange,
                    onRelayout: this.handleChange
                }, other))
            );
        }
    }]);

    return OncoPrint;
}(_react.PureComponent);

exports.default = OncoPrint;


OncoPrint.propTypes = {

    /**
     * Input data, in CBioPortal format where each list entry is a dict
     * consisting of 'sample', 'gene', 'alteration', and 'type'
     */
    data: _propTypes2.default.array,

    // TODO: Add remove empty columns prop

    /**
     * Adjusts the padding (amount of whitespace) between two tracks.
     * Value is a ratio between 0 and 1.
     * Default of 0.05 or 5%. If set to 0 plot will look like a heatmap.
     */
    padding: _propTypes2.default.number,

    /**
     * If not null, will override the default OncoPrint colorscale.
     * Default OncoPrint colorscale same as CBioPortal implementation.
     * Make your own colrscale as a {'mutation': COLOR} dict.
     * Supported mutation keys in ['MISSENSE, 'INFRAME', 'FUSION',
     * 'AMP', 'GAIN', 'HETLOSS', 'HMODEL', 'UP', 'DOWN']
     * Note that this is NOT a standard plotly colorscale.
     */
    colorscale: _propTypes2.default.oneOfType([_propTypes2.default.bool, _propTypes2.default.object]),

    /**
     * Default color for the tracks, in common name, hex, rgb or rgba format.
     * If left blank, will default to a light grey rgb(190, 190, 190).
     */
    backgroundcolor: _propTypes2.default.string,

    /**
     *.Toogles whether or not to show a legend on the right side of the plot,
     * with mutation information.
     */
    range: _propTypes2.default.array,

    /**
     *.Toogles whether or not to show a legend on the right side of the plot,
     * with mutation information.
     */
    showlegend: _propTypes2.default.bool,

    /**
     *.Toogles whether or not to show a heatmap overview of the tracks.
     */
    showoverview: _propTypes2.default.bool,

    /**
     * Width of the OncoPrint.
     * Will disable auto-resizing of plots if set.
     */
    width: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]),

    /**
     * Width of the OncoPrint.
     * Will disable auto-resizing of plots if set.
     */
    height: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string])
};

OncoPrint.defaultProps = {
    // Data
    padding: 0.05,
    colorscale: null,
    backgroundcolor: 'rgb(190, 190, 190)',
    // Layout
    range: [null, null],
    showlegend: true,
    showoverview: true,
    // Other
    width: null,
    height: 500
};