'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var hasElementsInInterval = exports.hasElementsInInterval = function hasElementsInInterval(arr, func, lowerBound, upperBound) {
    // in: arr, an array sorted in increasing order of func
    //     func, a function that takes an element of sorted_list and returns a number
    //     lowerBound and upperBound: define a half-open interval [lowerBound, upperBound)
    // out: boolean, true iff there are any elements whose image under func is in [lowerBound, upperBound)
    var lowerCounter = 0;
    var upperCounter = arr.length;
    var middle = void 0,
        middle_val = void 0;

    while (true) {

        if (lowerCounter >= upperCounter) {
            return false;
        }

        middle = Math.floor((lowerCounter + upperCounter) / 2);
        middle_val = func(arr[middle]);

        if (middle_val >= upperBound) {
            upperCounter = middle;
        } else if (middle_val < lowerBound) {
            lowerCounter = middle + 1;
        } else {
            // otherwise, the middle value is inside the interval,
            // so there's at least one value inside the interval
            return true;
        }
    }
};

// PrecomputedComparator is similar to the OncoPrintJs implementation with
// three notable changes: rewritten with Flow/ES next and as a class, the input
// data structure is different, and no direction.

var PrecomputedComparator = function () {
    function PrecomputedComparator(data, comparator) {
        _classCallCheck(this, PrecomputedComparator);

        this.comparator = comparator;
        this.data = data;

        this.sort();
    }

    _createClass(PrecomputedComparator, [{
        key: 'sort',
        value: function sort() {
            this.sortedData = this.data.sort(this.comparator);

            this.changePoints = [];
            for (var i = 0; i < this.sortedData.length; i += 1) {
                if (i === this.sortedData.length - 1) {
                    break;
                }

                if (this.comparator(this.sortedData[i], this.sortedData[i + 1]) !== 0) {
                    this.changePoints.push(i);
                }
            }

            this.samplesToIndex = {};
            for (var _i = 0; _i < this.sortedData.length; _i += 1) {
                this.samplesToIndex[this.sortedData[_i]] = _i;
            }
        }
    }, {
        key: 'compare',
        value: function compare(s1, s2) {
            var i1 = this.samplesToIndex[s1];
            var i2 = this.samplesToIndex[s2];

            if (typeof i1 === 'undefined' && typeof i2 === 'undefined') {
                return 0;
            } else if (typeof i1 === 'undefined') {
                return 1;
            } else if (typeof i2 === 'undefined') {
                return -1;
            }

            var shouldNegateResult = false;

            if (i1 === i2) {
                return 0;
            } else if (i1 > i2) {
                var tmp = i1;
                i1 = i2;
                i2 = tmp;
                shouldNegateResult = true;
            }

            var res = 0;
            if (hasElementsInInterval(this.changePoints, function (x) {
                return x;
            }, i1, i2)) {
                res = -1;
            }

            if (shouldNegateResult) {
                res *= -1;
            }

            return res;
        }
    }]);

    return PrecomputedComparator;
}();

exports.default = PrecomputedComparator;